# Generated by sila2.code_generator; sila2.__version__: 0.9.2
from __future__ import annotations

from abc import ABC, abstractmethod
from queue import Queue
from typing import TYPE_CHECKING, Optional

from sila2.server import FeatureImplementationBase, MetadataDict

from .huberchillercontrolservice_types import (
    ChangeSetPoint_Responses,
    ClearAlarmStatus_Responses,
    SetTemperatureControlMode_Responses,
    Temperature,
    TemperatureControlMode,
)

if TYPE_CHECKING:
    from ...server import Server


class HuberChillerControlServiceBase(FeatureImplementationBase, ABC):
    parent_server: Server

    _SetPointRangeLimit_producer_queue: Queue[RangeLimit]

    _SetPointValue_producer_queue: Queue[Temperature]

    _InternalActualValue_producer_queue: Queue[Temperature]

    _ExternalActualValue_producer_queue: Queue[Temperature]

    _TemperatureControlMode_producer_queue: Queue[TemperatureControlMode]

    _AlarmStatus_producer_queue: Queue[bool]

    def __init__(self, parent_server: Server):
        """
        Allows to control Huber Chiller devices
        """
        super().__init__(parent_server=parent_server)

        self._SetPointRangeLimit_producer_queue = Queue()

        self._SetPointValue_producer_queue = Queue()

        self._InternalActualValue_producer_queue = Queue()

        self._ExternalActualValue_producer_queue = Queue()

        self._TemperatureControlMode_producer_queue = Queue()

        self._AlarmStatus_producer_queue = Queue()

    @abstractmethod
    def get_OperatingRangeLimit(self, *, metadata: MetadataDict) -> RangeLimit:
        """
        The minimum and maximum operating temperatures of the device. These are specific to the device and cannot be changed.

        :param metadata: The SiLA Client Metadata attached to the call
        :return: The minimum and maximum operating temperatures of the device. These are specific to the device and cannot be changed.
        """
        pass

    def update_SetPointRangeLimit(self, SetPointRangeLimit: RangeLimit, queue: Optional[Queue[RangeLimit]] = None):
        """
        The minimum and maximum set point temperatures of the device. The minimum value cannot be below the minimum operating range limit and the maximum value cannot be higher than the maximum operating range limit.

        This method updates the observable property 'SetPointRangeLimit'.
        """
        if queue is None:
            queue = self._SetPointRangeLimit_producer_queue
        queue.put(SetPointRangeLimit)

    def SetPointRangeLimit_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[RangeLimit]]:
        """
        The minimum and maximum set point temperatures of the device. The minimum value cannot be below the minimum operating range limit and the maximum value cannot be higher than the maximum operating range limit.

        This method is called when a client subscribes to the observable property 'SetPointRangeLimit'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_SetPointValue(self, SetPointValue: Temperature, queue: Optional[Queue[Temperature]] = None):
        """
        The current set point value of the device.

        This method updates the observable property 'SetPointValue'.
        """
        if queue is None:
            queue = self._SetPointValue_producer_queue
        queue.put(SetPointValue)

    def SetPointValue_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Temperature]]:
        """
        The current set point value of the device.

        This method is called when a client subscribes to the observable property 'SetPointValue'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_InternalActualValue(self, InternalActualValue: Temperature, queue: Optional[Queue[Temperature]] = None):
        """
        The current actual value of the device. This value is measured by the internal temperature sensor of the device.

        This method updates the observable property 'InternalActualValue'.
        """
        if queue is None:
            queue = self._InternalActualValue_producer_queue
        queue.put(InternalActualValue)

    def InternalActualValue_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Temperature]]:
        """
        The current actual value of the device. This value is measured by the internal temperature sensor of the device.

        This method is called when a client subscribes to the observable property 'InternalActualValue'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_ExternalActualValue(self, ExternalActualValue: Temperature, queue: Optional[Queue[Temperature]] = None):
        """
        The current actual value of the device. This value is measured by the external temperature sensor of the device.

        This method updates the observable property 'ExternalActualValue'.
        """
        if queue is None:
            queue = self._ExternalActualValue_producer_queue
        queue.put(ExternalActualValue)

    def ExternalActualValue_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Temperature]]:
        """
        The current actual value of the device. This value is measured by the external temperature sensor of the device.

        This method is called when a client subscribes to the observable property 'ExternalActualValue'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_TemperatureControlMode(
        self, TemperatureControlMode: TemperatureControlMode, queue: Optional[Queue[TemperatureControlMode]] = None
    ):
        """
        The current temperature control mode (i.e. how the device should operate to reach the set point).

        This method updates the observable property 'TemperatureControlMode'.
        """
        if queue is None:
            queue = self._TemperatureControlMode_producer_queue
        queue.put(TemperatureControlMode)

    def TemperatureControlMode_on_subscription(
        self, *, metadata: MetadataDict
    ) -> Optional[Queue[TemperatureControlMode]]:
        """
        The current temperature control mode (i.e. how the device should operate to reach the set point).

        This method is called when a client subscribes to the observable property 'TemperatureControlMode'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_AlarmStatus(self, AlarmStatus: bool, queue: Optional[Queue[bool]] = None):
        """
        Indicates if the device is currently in an alarm state (true) or not (false). To clear the alarm state use the ClearAlarm command.

        This method updates the observable property 'AlarmStatus'.
        """
        if queue is None:
            queue = self._AlarmStatus_producer_queue
        queue.put(AlarmStatus)

    def AlarmStatus_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[bool]]:
        """
        Indicates if the device is currently in an alarm state (true) or not (false). To clear the alarm state use the ClearAlarm command.

        This method is called when a client subscribes to the observable property 'AlarmStatus'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    @abstractmethod
    def ChangeSetPoint(self, SetPointValue: Temperature, *, metadata: MetadataDict) -> ChangeSetPoint_Responses:
        """
        Change the set point value


        :param SetPointValue: The new set point value

        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass

    @abstractmethod
    def SetTemperatureControlMode(
        self, TemperatureControlMode: TemperatureControlMode, *, metadata: MetadataDict
    ) -> SetTemperatureControlMode_Responses:
        """
        Set the temperature control mode


        :param TemperatureControlMode: The temperature control mode to set.

        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass

    @abstractmethod
    def ClearAlarmStatus(self, *, metadata: MetadataDict) -> ClearAlarmStatus_Responses:
        """
        Clears the alarm state of the device if it is in an alarm state. Has no effect if the device is currently not in an alarm state.


        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass
