# Generated by sila2.code_generator; sila2.__version__: 0.8.0
from __future__ import annotations

from abc import ABC, abstractmethod
from queue import Queue
from typing import TYPE_CHECKING, Optional

from sila2.server import FeatureImplementationBase, MetadataDict, ObservableCommandInstance

from .continuousflowdosingservice_types import GenerateFlow_Responses, StopDosage_Responses

if TYPE_CHECKING:
    from sila2.server import SilaServer


class ContinuousFlowDosingServiceBase(FeatureImplementationBase, ABC):

    _MaxFlowRate_producer_queue: Queue[float]

    _FlowRate_producer_queue: Queue[float]

    def __init__(self, parent_server: SilaServer):
        """

                Allows to continuously dose a specified fluid.
        The continuous flow mode is meant for dispensing volumes or generating flows and works only in one direction. That means using negative flow rates or negative volumes for aspiration is not possible.

        """
        super().__init__(parent_server=parent_server)

        self._MaxFlowRate_producer_queue = Queue()

        self._FlowRate_producer_queue = Queue()

    def update_MaxFlowRate(self, MaxFlowRate: float, queue: Optional[Queue[float]] = None):
        """
        The maximum value of the flow rate at which this pump can dose a fluid.

        This method updates the observable property 'MaxFlowRate'.
        """
        if queue is None:
            queue = self._MaxFlowRate_producer_queue
        queue.put(MaxFlowRate)

    def MaxFlowRate_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[float]]:
        """
        The maximum value of the flow rate at which this pump can dose a fluid.

        This method is called when a client subscribes to the observable property 'MaxFlowRate'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_FlowRate(self, FlowRate: float, queue: Optional[Queue[float]] = None):
        """
        The current value of the flow rate. It is 0 if the pump does not dose any fluid.

        This method updates the observable property 'FlowRate'.
        """
        if queue is None:
            queue = self._FlowRate_producer_queue
        queue.put(FlowRate)

    def FlowRate_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[float]]:
        """
        The current value of the flow rate. It is 0 if the pump does not dose any fluid.

        This method is called when a client subscribes to the observable property 'FlowRate'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    @abstractmethod
    def StopDosage(self, *, metadata: MetadataDict) -> StopDosage_Responses:
        """
        Stops a currently running dosage immediately.


        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass

    @abstractmethod
    def GenerateFlow(
        self, FlowRate: float, *, metadata: MetadataDict, instance: ObservableCommandInstance
    ) -> GenerateFlow_Responses:
        """
        Generate a continuous flow with the given flow rate. Dosing continues until it gets stopped manually by calling StopDosage.



        :param FlowRate: The flow rate at which the pump should dose the fluid. This value cannot be negative since dosing is meant to only work in one direction.


        :param metadata: The SiLA Client Metadata attached to the call
        :param instance: The command instance, enabling sending status updates to subscribed clients

        """
        pass
